from dataclasses import dataclass

from lightbus.message import EventMessage


@dataclass
class QuestionHandle:
    id: str


class KStream:
    REDIS_STREAM_API_NAME = "KickerStream"

    def __init__(self, event_transport, stream_id):
        self.event_transport = event_transport
        self.stream_id = stream_id

    async def send_event(self, event_dict):
        await self.event_transport.send_event(
            EventMessage(
                api_name=KStream.REDIS_STREAM_API_NAME,
                event_name=f"{self.stream_id}",
                id=f"{self.stream_id}",
                kwargs=event_dict,
            ),
            options={},
        )

    def consume(self, consumer_name: str):
        return self.event_transport.consume(
            [(KStream.REDIS_STREAM_API_NAME, self.stream_id)],
            consumer_name,
            error_queue=None,
        )

    async def acknowledge(self, msg):
        await self.event_transport.acknowledge(msg)



from lightbus.transports.redis.event import RedisEventTransport, StreamUse

from kickredis.registry import ObjectRegistry
from kickredis.config import kicker_config
from kickredis.shared import KStream


class KickerRuntime:
    def __init__(self):
        self._event_transport = RedisEventTransport(
            service_name="", consumer_name="", stream_use=StreamUse.PER_EVENT
        )
        self._object_registry = ObjectRegistry(kicker_config.redis)

    @property
    def object_registry(self):
        return self._object_registry

    @property
    def event_transport(self):
        return self._event_transport

    def get_kstream(self, stream_id):
        return KStream(self.event_transport, stream_id)


kicker_runtime = KickerRuntime()



import logging
import asyncio
import lightbus.api

from lightbus.transports.redis.event import RedisEventTransport
from kickredis.shared import QuestionHandle
from kickredis.runtime import kicker_runtime


logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s,%(msecs)d - %(name)s/%(threadName)s - %(levelname)s: %(message)s",
    datefmt="%H:%M:%S",
)

log = logging.getLogger("kicker.services")


class Question:
    def __init__(self, q_id: str):
        self.id = q_id

    def getHandle(self):
        return QuestionHandle(self.id)

    async def run(self, event_transport):
        i = 0
        kstream = kicker_runtime.get_kstream(self.id)
        while True:
            log.info(f"Question[{self.id}]:{i}")
            i += 1
            await kstream.send_event({"sensor1": i})
            await asyncio.sleep(1)


class AnalyticsService(lightbus.Api):
    event_transport = RedisEventTransport(service_name="", consumer_name="")

    def __init__(self):
        self.qn = 0
        self.questions_map = {}

    class Meta:
        name = "AnalyticsService"

    async def new_question(self, query, is_start) -> QuestionHandle:
        return await self.run_question()

    async def run_question(self):
        log.debug("run_question")
        q = Question(self._gen_id())
        task = asyncio.create_task(q.run(self.event_transport))
        self.questions_map[q.id] = task
        return q.getHandle()

    def _gen_id(self) -> str:
        self.qn += 1
        return f"q_id_{self.qn}"


async def runFlow():
    bus = lightbus.create()
    bus.client.register_api(AnalyticsService())
    await bus.client.start_worker()
    log.info("==== Kick Runtime Started ====")


def main():
    loop = asyncio.get_event_loop()

    try:
        loop.create_task(runFlow())
        loop.run_forever()
    except KeyboardInterrupt:
        logging.info("Process interrupted")
    finally:
        loop.close()
        logging.info("Successfully shutdown")


if __name__ == "__main__":
    # asyncio.run (main1(), debug=True)
    main()



from dataclasses import dataclass


@dataclass
class RedisConf:
    port: int = 6379
    host: str = "localhost"


@dataclass
class KConfig:
    redis: RedisConf


kicker_config = KConfig(RedisConf())



import asyncio
import logging
import lightbus
from kickredis.runtime import kicker_runtime

logging.basicConfig(
    level=logging.DEBUG,
    format="%(asctime)s,%(msecs)d - %(name)s/%(threadName)s - %(levelname)s: %(message)s",
    datefmt="%H:%M:%S",
)

log = logging.getLogger("kicker.client")

bus = lightbus.create()


async def create_question():
    qh = await bus.AnalyticsService.new_question.call_async(
        query="device(color == red).stream(stream1).window(count, 10).compute(average)",
        is_start=True,
    )
    return qh


async def consume_question_results(question_handle):
    question_id: str = question_handle["id"]
    log.info(f"Start listing for result for question id={question_id}")
    kstream = kicker_runtime.get_kstream(question_id)

    async for messages in kstream.consume("mqtt_bridge_consumer1"):
        for msg in messages:
            log.info(f"Processed message :{msg}")
            await kstream.acknowledge(msg)


async def consume_sensor_data():
    device_id = "bot_dev"
    dev = kicker_runtime.object_registry.find_device(device_id)

    temp_stream = dev.find_stream_by_name("temperature")

    async for messages in temp_stream.consume("mqtt_bridge_consumer2"):
        log.info(f"Got {len(messages)} messages")
        for msg in messages:
            log.info(f"Processed message :{msg}")
            await temp_stream.acknowledge(msg)


async def runFlow():
    await consume_sensor_data()
    # qHandle = await create_question()
    # qHandle = {"id": "q_id_1"}
    # log.info(f"Question created. Handle:{qHandle}")
    # await consume_question_results(qHandle)


def main():
    loop = asyncio.get_event_loop()

    try:
        loop.create_task(runFlow())
        loop.run_forever()
    except KeyboardInterrupt:
        logging.info("Process interrupted")
    finally:
        loop.close()
        logging.info("Successfully shutdown")


if __name__ == "__main__":
    main()
